package org.magmafoundation.bot;

import dev.JustRed23.abcm.Config;
import dev.JustRed23.jdautils.JDAUtilities;
import dev.JustRed23.jdautils.settings.DefaultGuildSettingManager;
import dev.JustRed23.stonebrick.app.Application;
import dev.JustRed23.stonebrick.data.FileStructure;
import dev.JustRed23.stonebrick.log.SBLogger;
import dev.JustRed23.stonebrick.version.GitVersion;
import gs.mclo.api.MclogsClient;
import me.duncte123.botcommons.BotCommons;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.OnlineStatus;
import net.dv8tion.jda.api.entities.Activity;
import net.dv8tion.jda.api.requests.GatewayIntent;
import net.dv8tion.jda.api.utils.ChunkingFilter;
import net.dv8tion.jda.api.utils.MemberCachePolicy;
import net.dv8tion.jda.api.utils.cache.CacheFlag;
import org.gitlab4j.api.GitLabApi;
import org.magmafoundation.bot.command.base.CommandRegistry;
import org.magmafoundation.bot.core.ColorParser;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.core.MagmaFS;
import org.magmafoundation.bot.core.MagmaToken;
import org.magmafoundation.bot.db.DatabaseManager;
import org.magmafoundation.bot.db.tables.ActivityTable;
import org.magmafoundation.bot.event.BaseEventListener;
import org.magmafoundation.bot.services.UpdateChecker;
import org.magmafoundation.bot.services.UpdateSlashCommands;
import org.slf4j.Logger;

import java.sql.SQLException;

public class AppMain extends Application {

    private static Logger LOGGER;
    private JDABuilder builder;
    private static JDA instance;
    private static GitVersion version;

    private static MclogsClient mclogsApi;
    private static GitLabApi gitLabApi;

    protected void init() throws Exception {
        LOGGER = SBLogger.getLogger(Magma.name);
        version = GitVersion.fromFile(getClass().getClassLoader().getResourceAsStream("application.properties"));
        FileStructure.discover(MagmaFS.class);

        DatabaseManager.init();

        if (MagmaToken.token.isBlank()) {
            LOGGER.error("Token is blank, please add a token to the config");
            System.exit(1);
        }

        if (!MagmaToken.gitlab.isBlank()) {
            gitLabApi = new GitLabApi(Magma.git_url, MagmaToken.gitlab);
        }

        builder = JDABuilder.createDefault(MagmaToken.token)
                .setBulkDeleteSplittingEnabled(false)
                .setEnableShutdownHook(false)
                .setEnabledIntents(
                        GatewayIntent.DIRECT_MESSAGES,
                        GatewayIntent.GUILD_MESSAGES,
                        GatewayIntent.GUILD_MESSAGE_REACTIONS,
                        GatewayIntent.GUILD_MEMBERS,
                        GatewayIntent.MESSAGE_CONTENT,
                        GatewayIntent.GUILD_EMOJIS_AND_STICKERS
                )
                .disableCache(
                        CacheFlag.ACTIVITY,
                        CacheFlag.VOICE_STATE,
                        CacheFlag.SCHEDULED_EVENTS
                )
                .setChunkingFilter(ChunkingFilter.ALL)
                .setMemberCachePolicy(MemberCachePolicy.ALL)
                .setStatus(getStatus())
                .setActivity(getActivity());

        mclogsApi = new MclogsClient(Magma.name + "#" + Magma.tag, version.gitHash());

        getServicePool().addService(UpdateChecker.class);
        getServicePool().addService(UpdateSlashCommands.class);
    }

    protected void start() throws Exception {
        if (!Magma.enabled) {
            LOGGER.info("Bot is disabled, make sure all the values in magma.cfg are set!");
            exit();
            return;
        }

        instance = builder.build();
        instance.addEventListener(JDAUtilities.getInstance().withGuildSettingManager(new DefaultGuildSettingManager()).listener(), new BaseEventListener());
        LOGGER.info("Waiting for JDA to be ready...");
        instance.awaitReady();
        LOGGER.info("JDA is ready!");
        CommandRegistry.init();
        UpdateSlashCommands.collectedSlashCommands = true;
    }

    protected void stop() throws Exception {
        if (instance == null || !Magma.enabled)
            return;
        BotCommons.shutdown(instance);
    }

    private OnlineStatus getStatus() {
        return OnlineStatus.DO_NOT_DISTURB;
    }

    private Activity getActivity() {
        if(Magma.useDatabase) {
            try {
                Object activity = DatabaseManager.getTable(ActivityTable.class).get();
                return Activity.streaming((activity != null ? ((ActivityTable.Activity) activity).activity() : "Magma Dev"), "https://www.twitch.tv/hexeptiondev");
            } catch (SQLException e) {
                LOGGER.error("Failed to get activity from database", e);
            }
        }

        return Activity.streaming("Magma Dev", "https://www.twitch.tv/hexeptiondev");
    }

    public static JDA getInstance() {
        return instance;
    }

    public static MclogsClient getMclogsApi() {
        return mclogsApi;
    }

    public static GitLabApi getGitLabApi() {
        return gitLabApi;
    }

    public static GitVersion getVersion() {
        return version;
    }

    public static void main(String[] args) {
        Config.addParser(ColorParser.class);
        Config.addScannable(AppMain.class.getPackage()); //gets scanned recursively
        Config.setDebug(true);
        launch(args);
    }
}
