package org.magmafoundation.bot.command.commands.utility;

import dev.JustRed23.stonebrick.log.SBLogger;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandData;
import org.apache.commons.validator.routines.UrlValidator;
import org.magmafoundation.bot.autoresponse.ResponseHandler;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.db.DatabaseManager;
import org.magmafoundation.bot.db.tables.ResponseTable;
import org.slf4j.Logger;

import java.sql.SQLException;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;

import static org.magmafoundation.bot.command.base.CheckTypes.SUCCESS;

public class AutoresponseCommand implements ICommand {

    private final Logger LOGGER = SBLogger.getLogger(AutoresponseCommand.class);

    public CheckTypes canUserExecute(CommandContext context) {
        Role staff = context.getJDA().getRoleById(Magma.staff_role_id);

        String subcommandName = context.getEvent().getSubcommandName();
        if (subcommandName != null && subcommandName.equalsIgnoreCase("list"))
            return context.getMember().getRoles().contains(staff) ? SUCCESS : CheckTypes.COMMAND_NO_PERMISSION;

        Role supermod = context.getJDA().getRoleById(Magma.super_mod_role_id);
        return context.getMember().getRoles().contains(supermod) ? SUCCESS : CheckTypes.COMMAND_NO_PERMISSION;
    }

    public void execute(CommandContext context) {
        if (!Magma.useDatabase) {
            context.getEvent().reply("Command not available, database is disabled").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (context.getEvent().getSubcommandName() == null) {
            context.getEvent().reply("Invalid subcommand!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        switch (context.getEvent().getSubcommandName()) {
            case "add" -> add(context);
            case "info" -> info(context);
            case "list" -> list(context);
            case "remove" -> remove(context);
        }
    }

    public void add(CommandContext context) {
        String trigger = context.getEvent().getOption("trigger").getAsString();
        String title = context.getEvent().getOption("title").getAsString();
        String response = context.getEvent().getOption("response").getAsString();
        String image = context.getEvent().getOption("image") != null ? context.getEvent().getOption("image").getAsString() : "";

        if (trigger == null || title == null || response == null) {
            context.getEvent().reply("Invalid arguments!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (trigger.trim().isBlank() || title.trim().isBlank() || response.trim().isBlank()) {
            context.getEvent().reply("Invalid arguments!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (trigger.length() > 50) {
            context.getEvent().reply("The trigger can't be longer than 50 characters!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (title.length() > 50) {
            context.getEvent().reply("The title can't be longer than 50 characters!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (response.length() > 1000) {
            context.getEvent().reply("The response can't be longer than 1000 characters!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (!image.isBlank()) {
            if (image.length() > 255) {
                context.getEvent().reply("The image url can't be longer than 255 characters!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
                return;
            }

            //try connecting to the image url
            boolean valid = UrlValidator.getInstance().isValid(image);

            if (!valid) {
                context.getEvent().reply("Invalid image url!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
                return;
            }
        }

        ResponseTable.Response resp = new ResponseTable.Response(trigger, title, response, image);

        try {
            DatabaseManager.getTable(ResponseTable.class).add(resp);
            ResponseHandler.clearCache();
            context.getEvent().reply("Added response '" + resp.trigger() + "'").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
        } catch (SQLException e) {
            LOGGER.error("Failed to add response to database", e);
            context.getEvent().reply("Failed to add response to database").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
        }
    }

    public void info(CommandContext context) {
        String trigger = context.getEvent().getOption("trigger").getAsString();

        Object responseList = null;

        try {
            responseList = DatabaseManager.getTable(ResponseTable.class).get();
        } catch (SQLException e) {
            LOGGER.error("Failed to get responses from database", e);
            context.getEvent().reply("Failed to get responses from database").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (responseList == null || ((List<ResponseTable.Response>) responseList).isEmpty()) {
            context.getEvent().reply("No responses found in database").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        List<ResponseTable.Response> responses = (List<ResponseTable.Response>) responseList;

        responses.stream().filter(response -> response.trigger().equalsIgnoreCase(trigger)).findFirst().ifPresentOrElse(response -> {
            EmbedBuilder embed = new EmbedBuilder();
            embed.setTitle("Autoresponse for '" + response.trigger() + "'");
            embed.addField("Title", response.title(), false);
            embed.addField("Response", response.response(), false);
            embed.setColor(Magma.embed_color);
            embed.setThumbnail(context.getJDA().getSelfUser().getAvatarUrl());
            context.getEvent().replyEmbeds(embed.build()).queue();
        }, () -> context.getEvent().reply("No response found for '" + trigger + "'").queue());
    }

    public void list(CommandContext context) {
        Object responseList = null;

        try {
            responseList = DatabaseManager.getTable(ResponseTable.class).get();
        } catch (SQLException e) {
            LOGGER.error("Failed to get responses from database", e);
            context.getEvent().reply("Failed to get responses from database").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (responseList == null || ((List<ResponseTable.Response>) responseList).isEmpty()) {
            context.getEvent().reply("No responses found in database").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        List<ResponseTable.Response> responses = (List<ResponseTable.Response>) responseList;

        EmbedBuilder embed = new EmbedBuilder();
        embed.setTitle("Showing list of all autoresponses");
        embed.setColor(Magma.embed_color);
        embed.setThumbnail(context.getJDA().getSelfUser().getAvatarUrl());

        for (ResponseTable.Response response : responses) {
            embed.addField(response.trigger(), response.title(), false);
        }

        context.getEvent().replyEmbeds(embed.build()).queue();
    }

    public void remove(CommandContext context) {
        String trigger = context.getEvent().getOption("trigger").getAsString();

        if (trigger == null) {
            context.getEvent().reply("Invalid arguments!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (trigger.trim().isBlank()) {
            context.getEvent().reply("Invalid arguments!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (trigger.length() > 50) {
            context.getEvent().reply("The trigger can't be longer than 50 characters!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        try {
            Object responseList = DatabaseManager.getTable(ResponseTable.class).get(trigger);
            if (responseList == null || ((List<ResponseTable.Response>) responseList).isEmpty()) {
                context.getEvent().reply("No responses found in database").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
                return;
            }

            AtomicBoolean found = new AtomicBoolean(false);
            ((List<ResponseTable.Response>) responseList).stream().filter(response -> response.trigger().equalsIgnoreCase(trigger)).findAny().ifPresent(response -> found.set(true));

            if (!found.get()) {
                context.getEvent().reply("No response found for '" + trigger + "'").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
                return;
            }

            DatabaseManager.getTable(ResponseTable.class).remove(trigger);
            ResponseHandler.clearCache();
            context.getEvent().reply("Removed response '" + trigger + "'").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
        } catch (SQLException e) {
            LOGGER.error("Failed to remove response from database", e);
            context.getEvent().reply("Failed to remove response from database").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
        }
    }

    public String name() {
        return "autoresponse";
    }

    public String help() {
        return "Used to manage autoresponses";
    }

    public CommandData commandData() {
        SubcommandData list = new SubcommandData("list", "List all autoresponses");
        SubcommandData add = new SubcommandData("add", "Add an autoresponse")
                .addOption(OptionType.STRING, "trigger", "The trigger for the autoresponse", true)
                .addOption(OptionType.STRING, "title", "The title for the autoresponse", true)
                .addOption(OptionType.STRING, "response", "The description of the autoresponse", true)
                .addOption(OptionType.STRING, "image", "The image for the autoresponse", false);


        SubcommandData remove = new SubcommandData("remove", "Remove an autoresponse")
                .addOption(OptionType.STRING, "trigger", "The trigger for the autoresponse", true);

        SubcommandData info = new SubcommandData("info", "Get info about an autoresponse")
                .addOption(OptionType.STRING, "trigger", "The trigger for the autoresponse", true);

        return Commands.slash("autoresponse", "Manage autoresponses")
                .addSubcommands(add, info, list, remove)
                .setGuildOnly(true);
    }

    public Category category() {
        return Category.UTILITY;
    }
}
