package org.magmafoundation.bot.command.commands.utility;

import dev.JustRed23.stonebrick.exceptions.NetRequestException;
import dev.JustRed23.stonebrick.net.Callback;
import dev.JustRed23.stonebrick.net.NetworkManager;
import dev.JustRed23.stonebrick.net.Request;
import dev.JustRed23.stonebrick.net.Response;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandData;
import org.json.JSONArray;
import org.json.JSONObject;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;

public class SpigotMCCommand implements ICommand {

   private static int DEFAULT_SEARCH_AMOUNT = 10;

   private static final String SPIGET_SEARCH = "https://api.spiget.org/v2/search/resources/%s";
   private static final String SPIGET_SEARCH_AMOUNT = SPIGET_SEARCH + "?size=%d";
   private static final String SPIGET_INFO = "https://api.spiget.org/v2/resources/%s";
   private static final String SPIGOT_BASE_LINK = "https://spigotmc.org/%s";

   @Override
   public CheckTypes canUserExecute(CommandContext context) {
      return CheckTypes.SUCCESS;
   }

   @Override
   public void execute(CommandContext context) {
      if (context.getEvent().getSubcommandName() == null) {
         context.getEvent().reply("Please specify a subcommand").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, java.util.concurrent.TimeUnit.SECONDS));
         return;
      }

      switch (context.getEvent().getSubcommandName()) {
         case "list" -> handleSearch(context);
         case "display" -> handleDisplay(context);
         default -> context.getEvent().reply("Unknown subcommand").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, java.util.concurrent.TimeUnit.SECONDS));
      }
   }

   private void handleDisplay(CommandContext context) {

      var nameOption = context.getEvent().getOption("plugin-id");

      NetworkManager.get(SPIGET_INFO.formatted(nameOption.getAsString())).async(new Callback() {
         @Override
         public void response(Request request, Response response, NetRequestException e) {
            if (e != null) {
               context.getEvent().reply("An error occurred while fetching the stats: " + e.getMessage()).queue();
               return;
            }

            JSONObject json = response.asJSONObject();

            String embedTitle = "%s (%d Downloads)".formatted(json.getString("name"), json.getInt("downloads"));

            EmbedBuilder embed = new EmbedBuilder();
            embed.setTitle(embedTitle);

            // Short description
            embed.addField("Description", json.getString("tag"), true);

            // Short description
            embed.addField("Rating", json.getJSONObject("rating").getInt("average") + " :star:", true);

            // Supported versions
            StringBuilder stringBuilder = new StringBuilder();
            JSONArray versions = (JSONArray) json.get("testedVersions");
            versions.forEach(version -> stringBuilder.append(version + " "));

            String url = SPIGOT_BASE_LINK.formatted(json.getJSONObject("file").getString("url"));
            url = url.substring(0, url.indexOf("/download") + 1); //make sure we keep the /
            if (versions.isEmpty())
                stringBuilder.append("No version provided, check the [spigot page](%s) for more information".formatted(url));

            embed.addField("Versions", stringBuilder.toString(), false);

            embed.setColor(Magma.embed_color);
            embed.setFooter("Stats by https://spiget.org");
            String imageUrl = json.getJSONObject("icon").getString("url");
            if (!imageUrl.isEmpty())
               embed.setImage(SPIGOT_BASE_LINK.formatted(imageUrl));

            context.getEvent().replyEmbeds(embed.build()).queue();
         }
      });
   }

   private void handleSearch(CommandContext context) {

      var nameOption = context.getEvent().getOption("query");
      var amountOption = context.getEvent().getOption("amount");
      int amount = amountOption != null ? amountOption.getAsInt() : 10;

      NetworkManager.get(SPIGET_SEARCH_AMOUNT.formatted(nameOption.getAsString(), amount)).async(new Callback() {
         @Override
         public void response(Request request, Response response, NetRequestException e) {
            if (e != null) {
               context.getEvent().reply("An error occurred while fetching the stats").queue();
               return;
            }

            JSONArray json = response.asJSONArray();

            String embedTitle = "SpigotMC Search for `%s` (%d entries)".formatted(nameOption.getAsString(), amount);

            EmbedBuilder embed = new EmbedBuilder();
            embed.setTitle(embedTitle);

            for (Object obj : json) {
               JSONObject entry = (JSONObject) obj;
               int id = entry.getInt("id");
               var downloadLink = SPIGOT_BASE_LINK.formatted("resources/" + id);
               embed.addField("%s (ID: %d)".formatted(entry.getString("name"), id), "[(%d Downloads)](%s)".formatted(entry.getInt("downloads"), downloadLink), false);
            }

            embed.setColor(Magma.embed_color);
            embed.setFooter("Stats by https://spiget.org");

            context.getEvent().replyEmbeds(embed.build()).queue();
         }
      });
   }

   @Override
   public String name() {
      return "spigotmc";
   }

   @Override
   public String help() {
      return "Search and display plugins from SpigotMC.org";
   }

   @Override
   public CommandData commandData() {
      SubcommandData display = new SubcommandData("display", "Display a plugin from SpigotMC.org")
         .addOption(OptionType.INTEGER, "plugin-id", "The ID of the plugin (use '/" + name() + " list' to get it)", true);

      SubcommandData list = new SubcommandData("list", "List plugins matching your query")
            .addOption(OptionType.STRING, "query", "The plugin name", true)
            .addOption(OptionType.INTEGER, "amount", "The amount of plugins to display", false);

      return Commands.slash(name(), help())
              .addSubcommands(display, list)
              .setGuildOnly(true);
   }

   @Override
   public Category category() {
      return Category.UTILITY;
   }
}
