package org.magmafoundation.bot.log;

import dev.JustRed23.stonebrick.log.SBLogger;
import gs.mclo.api.Log;
import gs.mclo.api.response.InsightsResponse;
import gs.mclo.api.response.UploadLogResponse;
import gs.mclo.api.response.insights.Analysis;
import gs.mclo.api.response.insights.Problem;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.magmafoundation.bot.AppMain;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.db.DatabaseManager;
import org.magmafoundation.bot.db.tables.LogUploadTable;
import org.magmafoundation.bot.utils.FileUtils;
import org.slf4j.Logger;

import java.awt.*;
import java.io.IOException;
import java.sql.SQLException;

public class LogUploader {

    public static boolean enabled = true;

    public static void checkEnabled() {
        if (!Magma.useDatabase)
            return;

        LogUploadTable.LogUpload upload;
        try {
            upload = (LogUploadTable.LogUpload) DatabaseManager.getTable(LogUploadTable.class).get();
        } catch (SQLException e) {
            LOGGER.error("Failed to check if log upload is enabled", e);
            return;
        }

        if (upload == null)
            return;

        enabled = upload.enabled();
    }

    private static final Logger LOGGER = SBLogger.getLogger(LogUploader.class);

    public static void uploadLog(Message.Attachment attachment, MessageReceivedEvent event) {
        TextChannel channel = event.getGuildChannel().asTextChannel();

        try (attachment) {
            if (Log.ALLOWED_FILE_NAME_PATTERN.matcher(attachment.getFileName()).matches()) {
                if (attachment.getSize() > Log.MAX_LOG_LENGTH)
                    channel.sendMessage(":warning: File exceeds " + ((Log.MAX_LOG_LENGTH / 1024) / 1024) + "MB, your log might be shortened!").queue();

                attachment.getProxy().download().thenAcceptAsync(file -> {
                    String content = null;

                    try (file) {
                        byte[] bytes = file.readAllBytes();
                        content = new String(bytes);
                    } catch (Exception e) {
                        channel.sendMessage("Could not read file. Corrupt?").queue();
                    }

                    if (content == null || content.isBlank())
                        return;

                    EmbedBuilder builder = doChecks(content);

                    if (builder != null) {
                        channel.sendMessageEmbeds(builder.build()).queue();
                        return;
                    }

                    Log log = new Log(content);
                    try {
                        final UploadLogResponse response = AppMain.getMclogsApi().uploadLog(log);

                        if (response.getError() != null && !response.getError().isBlank()) {
                            channel.sendMessage("Could not upload log to mclo.gs: " + response.getError()).queue();
                            return;
                        }

                        if (response.isSuccess()) {
                            EmbedBuilder uploaded = new EmbedBuilder();
                            uploaded.setTitle("Log uploaded to mclo.gs");
                            uploaded.setColor(Magma.embed_color);
                            uploaded.setThumbnail("https://media.discordapp.net/attachments/815715785091186719/1027314416468901988/mclogs.png");

                            StringBuilder sb = new StringBuilder();
                            sb.append("**- Title: **").append(attachment.getFileName()).append("\n");
                            sb.append("**- Size: **").append(FileUtils.getFileSize(attachment.getSize())).append("\n");
                            sb.append("**- Link: **").append(response.getUrl());

                            StringBuilder insights = getInsights(response);
                            if (insights != null)
                                sb.append("\n").append(insights);

                            uploaded.setDescription(sb.toString());
                            uploaded.setFooter("File %s sent by %s".formatted(attachment.getFileName(), event.getAuthor().getAsTag()), event.getAuthor().getEffectiveAvatarUrl());

                            channel.sendMessageEmbeds(uploaded.build()).queue();
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                        channel.sendMessage("Could not upload log to mclo.gs: " + e.getMessage()).queue();
                    }
                });
            }
        }
    }

    private static EmbedBuilder doChecks(String content) {
        EmbedBuilder error = new EmbedBuilder();
        error.setColor(Color.RED);
        error.setThumbnail("https://discord.com/assets/22841a994e9477aaf121a8844c07ebf0.svg"); //no-entry emote

        if (content.contains("SERVER IS RUNNING IN OFFLINE/INSECURE MODE!") && !(content.contains("Whilst this makes it possible to use BungeeCord") || content.contains("Whilst this makes it possible to use Velocity"))) {
            error.setTitle("Detected offline mode!");
            error.setDescription("We do not support offline mode (online-mode=false) as it encourages piracy of the game and also introduces some security risks. We strongly discourage using it and will not offer **any** support to users who do!");
            return error;
        }

        if (content.contains("Minecraft Version: 1.16.5") || content.contains("--fml.mcVersion, 1.16.5")) {
            error.setTitle("Detected deprecated version!");
            error.setDescription("We will not provide support for Magma 1.16.5 anymore as it has been deprecated. Please update to a newer version of Magma.");
            return error;
        }

        return null;
    }

    private static @Nullable StringBuilder getInsights(@NotNull UploadLogResponse resp) {
        try {
            final InsightsResponse insights = resp.getInsights();

            StringBuilder sb = new StringBuilder();
            sb.append("\n**__Insights__**").append("\n");

            if (insights.getError() != null && !insights.getError().isBlank()) {
                LOGGER.error("Could not get insights for log {}: {}", resp.getId(), insights.getError());
                return null;
            }

            final Analysis analysis = insights.getAnalysis();
            if (analysis == null)
                return null;

            if (analysis.getProblems() == null || analysis.getProblems().length == 0)
                return null;

            for (int pI = 0; pI < analysis.getProblems().length; pI++) {
                final Problem problem = analysis.getProblems()[pI];
                sb.append(":warning: ").append(problem.getMessage());
                if (pI != analysis.getProblems().length - 1)
                    sb.append("\n");
            }

            return sb;
        } catch (IOException e) {
            LOGGER.error("Could not get insights for log {}", resp.getId(), e);
            return null;
        }
    }
}
